#include "roaddlg.h"
#include "AppMain.h"
#include <wx/sstream.h>

roaddlg::roaddlg( wxWindow* parent )
:
dlgbase( parent )
{

}

void roaddlg::BkgdClear() {
	this->bkgdname->Clear();
	this->bkgdbegl->Clear();
	this->bkgdbegr->Clear();
	this->bkgdendl->Clear();
	this->bkgdendr->Clear();
}

void roaddlg::FrgdClear() {
	this->frgdname->Clear();
	this->frgdbeg->Clear();
	this->frgdend->Clear();
	this->frgdwidth->Clear();
	this->peron->Clear();
	this->peroff->Clear();
	this->perstart->Clear();
}

void roaddlg::OtherClear() {
	this->grpname->Clear();
	this->patternlength->Clear();
}

void roaddlg::OnClose( wxCloseEvent&  )
{
	wxTheApp->Exit();
}

void roaddlg::OnInitDialog( wxInitDialogEvent&  )
{
	this->frgdpicker->SetColour(*wxWHITE);
}

void roaddlg::OnClear(wxCommandEvent&) 
{
	BkgdClear();
	FrgdClear();
	this->bkgdchoices->SetSelection(wxNOT_FOUND);
	this->frgdchoices->SetSelection(wxNOT_FOUND);
}

void roaddlg::BkgdColSet(wxColourPickerEvent&) 
{
	wxColour colour = this->bkgdpicker->GetColour();
	wxInt32 rgba = colour.GetRGBA() | 0xff000000;
	colour.SetRGBA(rgba);
	this->bkgdpicker->SetColour(colour);
}

void roaddlg::BkgdSel(wxCommandEvent&) 
{
	int position = this->bkgdchoices->GetSelection();
	if (position == wxNOT_FOUND) return;
	auto i = bkgd_items.begin() + position;
	
	this->bkgdname->SetValue(i->name);
	this->bkgdpicker->SetColour(i->rgb);
	this->bkgdbegl->SetValue(wxString::FromCDouble(i->start_left));
	this->bkgdbegr->SetValue(wxString::FromCDouble(i->start_right));
	this->bkgdendl->SetValue(wxString::FromCDouble(i->end_left));
	this->bkgdendr->SetValue(wxString::FromCDouble(i->end_right));
}

void roaddlg::BkgdInsert(wxCommandEvent&)
{
	wxString temp;
	bkgd_item temp2;
	try {
		temp2.name = this->bkgdname->GetValue();

		temp2.rgb = this->bkgdpicker->GetColour();

		temp = this->bkgdbegl->GetValue();
		if (!temp.ToCDouble(&temp2.start_left)) throw(0);
		temp = this->bkgdbegr->GetValue();
		if (!temp.ToCDouble(&temp2.start_right)) throw(0);

		temp = this->bkgdendl->GetValue();
		if (temp.IsEmpty()) {
			temp2.end_left = temp2.start_left;
		}
		else {
			if (!temp.ToCDouble(&temp2.end_left)) throw(0);
		}

		temp = this->bkgdendr->GetValue();
		if (temp.IsEmpty()) {
			temp2.end_right = temp2.start_right;
		}
		else {
			if (!temp.ToCDouble(&temp2.end_right)) throw(0);
		}
	}
	catch (int) {
		wxMessageBox("Invalid floating point format!");
	}

	int position = this->bkgdchoices->GetSelection();
	if (position == wxNOT_FOUND) position = static_cast<signed>(this->bkgdchoices->GetCount());
	auto i = bkgd_items.begin() + position;
	bkgd_items.insert(i, temp2);
	this->bkgdchoices->InsertItems(1, &temp2.name, position);

	BkgdClear();
}

void roaddlg::BkgdModify( wxCommandEvent&  )
{
	try {
		int position = this->bkgdchoices->GetSelection();
		if (position == wxNOT_FOUND) throw("No item selected to modify");
		auto i = this->bkgd_items.begin() + position;
		wxString temp;
		i->name = this->bkgdname->GetValue();

		i->rgb = this->bkgdpicker->GetColour();

		temp = this->bkgdbegl->GetValue();
		if (!temp.ToCDouble(&i->start_left)) throw("Invalid floating point format!");
		temp = this->bkgdbegr->GetValue();
		if (!temp.ToCDouble(&i->start_right)) throw("Invalid floating point format!");

		temp = this->bkgdendl->GetValue();
		if (temp.IsEmpty()) {
			i->end_left = i->start_left;
		}
		else {
			if (!temp.ToCDouble(&i->end_left)) throw("Invalid floating point format!");
		}

		temp = this->bkgdendr->GetValue();
		if (temp.IsEmpty()) {
			i->end_right = i->start_right;
		}
		else {
			if (!temp.ToCDouble(&i->end_right)) throw("Invalid floating point format!");
		}

		this->bkgdchoices->SetString(static_cast<unsigned>(position), i->name);
	}
	catch (const char* err) {
		wxMessageBox(err);
	}

	BkgdClear();
}

void roaddlg::BkgdRemove( wxCommandEvent&  )
{
	int position = this->bkgdchoices->GetSelection();
	if (position == wxNOT_FOUND) return;
	this->bkgdchoices->Delete(static_cast<unsigned>(position));
	auto i = bkgd_items.begin() + position;
	bkgd_items.erase(i);

	void BkgdClear();
}

void roaddlg::FrgdColSet(wxColourPickerEvent&) {
	wxColour colour = this->frgdpicker->GetColour();
	wxInt32 rgba = colour.GetRGBA() | 0xff000000;
	colour.SetRGBA(rgba);
	this->frgdpicker->SetColour(colour);
}

void roaddlg::FrgdSel(wxCommandEvent&) 
{
	int position = this->frgdchoices->GetSelection();
	if (position == wxNOT_FOUND) return;
	auto i = frgd_items.begin() + position;

	this->frgdname->SetValue(i->name);
	this->frgdpicker->SetColour(i->rgb);
	this->frgdbeg->SetValue(wxString::FromCDouble(i->start_left));
	this->frgdend->SetValue(wxString::FromCDouble(i->end_left));
	this->frgdwidth->SetValue(wxString::FromCDouble(i->width));
	this->peron->SetValue(wxString::FromCDouble(i->per_on));
	this->peroff->SetValue(wxString::FromCDouble(i->per_off));
	this->perstart->SetValue(wxString::FromCDouble(i->per_start));
}

void roaddlg::FrgdInsert(wxCommandEvent&)
{
	wxString temp;
	frgd_item temp2;
	try {
		temp2.name = this->frgdname->GetValue();

		temp2.rgb = this->frgdpicker->GetColour();

		temp = this->frgdbeg->GetValue();
		if (!temp.ToCDouble(&temp2.start_left)) throw(0);

		temp = this->frgdend->GetValue();
		if (temp.IsEmpty()) {
			temp2.end_left = temp2.start_left;
		}
		else {
			if (!temp.ToCDouble(&temp2.end_left)) throw(0);
		}

		temp = this->frgdwidth->GetValue();
		if (!temp.ToCDouble(&temp2.width)) throw(0);

		temp = this->peron->GetValue();
		if (!temp.ToCDouble(&temp2.per_on)) throw(0);

		temp = this->peroff->GetValue();
		if (!temp.ToCDouble(&temp2.per_off)) throw(0);

		temp = this->perstart->GetValue();
		if (!temp.ToCDouble(&temp2.per_start)) throw(0);

	}
	catch (int) {
		wxMessageBox("Invalid floating point format!");
	}

	int position = this->frgdchoices->GetSelection();
	if (position == wxNOT_FOUND) position = static_cast<signed>(this->frgdchoices->GetCount());
	auto i = frgd_items.begin() + position;
	frgd_items.insert(i, temp2);
	this->frgdchoices->InsertItems(1, &temp2.name, static_cast<unsigned>(position));

	FrgdClear();
}

void roaddlg::FrgdModify( wxCommandEvent&  )
{
	try {
		int position = this->frgdchoices->GetSelection();
		if (position == wxNOT_FOUND) throw("No item selected to modify");
		auto i = frgd_items.begin() + position;
		wxString temp;

		i->name = this->frgdname->GetValue();

		i->rgb = this->frgdpicker->GetColour();

		temp = this->frgdbeg->GetValue();
		if (!temp.ToCDouble(&i->start_left)) throw("Invalid floating point format!");

		temp = this->frgdend->GetValue();
		if (temp.IsEmpty()) {
			i->end_left = i->start_left;
		}
		else {
			if (!temp.ToCDouble(&i->end_left)) throw("Invalid floating point format!");
		}

		temp = this->frgdwidth->GetValue();
		if (!temp.ToCDouble(&i->width)) throw("Invalid floating point format!");

		temp = this->peron->GetValue();
		if (!temp.ToCDouble(&i->per_on)) throw("Invalid floating point format!");

		temp = this->peroff->GetValue();
		if (!temp.ToCDouble(&i->per_off)) throw("Invalid floating point format!");

		temp = this->perstart->GetValue();
		if (!temp.ToCDouble(&i->per_start)) throw("Invalid floating point format!");

		this->frgdchoices->SetString(static_cast<unsigned>(position), i->name);
	}
	catch (const char* err) {
		wxMessageBox(err);
	}

	FrgdClear();
}

void roaddlg::FrgdRemove( wxCommandEvent&  )
{
	int position = this->frgdchoices->GetSelection();
	if (position == wxNOT_FOUND) return;
	this->frgdchoices->Delete(static_cast<unsigned>(position));
	auto i = frgd_items.begin() + position;
	frgd_items.erase(i);

	FrgdClear();
}

void roaddlg::Generate( wxCommandEvent&  )
{
	wxString temp;
	group_info group;
	try {
		group.name = this->grpname->GetValue();

		temp = this->patternlength->GetValue();
		if (!temp.ToCDouble(&group.length)) throw(0);

		temp = this->startingy->GetValue();
		if (temp.IsEmpty()) {
			group.startingy;
		}
		else {
			if (!temp.ToCDouble(&group.startingy)) throw(0);
		}
	}
	catch (int) {
		wxMessageBox("Invalid floating point format!");
	}

	OtherClear();

	wxXmlDocument xml;
	xml.SetRoot(group.get_node());
	wxXmlNode* root = xml.GetRoot();
	for (auto& x : bkgd_items) {
		root->AddChild(x.get_node(group.startingy, group.length));
	}
	for (auto& x : frgd_items) {
		root->AddChild(x.get_node(group.startingy, group.length));
	}

	wxString out;
	wxStringOutputStream sstream(&out);
	xml.Save(sstream);

	output->SetValue(out);
}
